@@ -19,7 +19,7 @@ class Completion(EngineAPIResource, ListableAPIResource, DeletableAPIResource):
         of valid parameters.
         """
         start = time.time()
-        timeout = kwargs.get("timeout", None)
+        timeout = kwargs.pop("timeout", None)
         if kwargs.get("model", None) is None and kwargs.get("engine", None) is None:
             raise InvalidRequestError(
                 "Must provide an 'engine' or 'model' parameter to create a Completion.",
@@ -33,4 +33,4 @@ class Completion(EngineAPIResource, ListableAPIResource, DeletableAPIResource):
                 if timeout is not None and time.time() > start + timeout:
                     raise
 
-                util.log_info("Waiting for snapshot to warm up", error=e)
+                util.log_info("Waiting for model to warm up", error=e)
